
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#include "StdAfx.h"
#include "RangeAddress.h"

using namespace AutomateIT::CalcItems;

CRangeAddress::CRangeAddress( const std::wstring& name ) :
    topLeft_(0),
    bottomRight_(0)
{
    ConstructByName(name);
}


CRangeAddress::CRangeAddress( const CPoint& topLeft, const CPoint& bottomRight ) :
    topLeft_(new CAddressItem(topLeft)),
    bottomRight_(new CAddressItem(bottomRight))
{
    
}

void CRangeAddress::ConstructByName( const std::wstring& name )
{
    std::wstring processedString = name;

    size_t exPos = processedString.find(ex_);
    if (std::wstring::npos != exPos)
    {
        sheetName_ = processedString.substr(0, exPos);
        processedString = processedString.substr(exPos + 1);
    }

    size_t spotPos = processedString.find(spot_);
    if (std::wstring::npos == spotPos)
    {
        processedString = processedString + spot_ + processedString;
        spotPos = processedString.find(spot_);
    }

    std::wstring leftStr(processedString.substr(0, spotPos));
    std::wstring rightStr(processedString.substr(spotPos + 1, processedString.size() - spotPos - 1));

    if (CAddressItem::HasBothComponents(leftStr))
    {
        topLeft_.reset(new CAddressItem(leftStr));
    }
    else
    {
        topLeft_.reset(new CAddressItem(leftStr, CAddressItem::First));
    }

    if (CAddressItem::HasBothComponents(rightStr))
    {
        bottomRight_.reset(new CAddressItem(rightStr));
    }
    else
    {
        bottomRight_.reset(new CAddressItem(rightStr, CAddressItem::Second));
    }
}
const CPoint& CRangeAddress::GetTopLeft() const
{
    return topLeft_->GetAddress();
}

const CPoint& CRangeAddress::GetBottomRight() const
{
    return bottomRight_->GetAddress();
}

long CRangeAddress::GetLeft() const
{
    return topLeft_->GetAddress().x;
}

long CRangeAddress::GetRight() const
{
    return bottomRight_->GetAddress().x;
}

long CRangeAddress::GetTop() const
{
    return topLeft_->GetAddress().y;
}

long CRangeAddress::GetBottom() const
{
    return bottomRight_->GetAddress().y;
}

CRangeAddress::~CRangeAddress()
{

}

std::wstring CRangeAddress::ToString() const
{
    std::wstring prefix;
    if (!sheetName_.empty())
    {
        prefix += sheetName_ + L".";
    }
    return prefix + topLeft_->ToString() + spot_ + bottomRight_->ToString();
}

CRangeAddress& CRangeAddress::operator+=( const CPoint& point )
{
    (*topLeft_) += point;
    (*bottomRight_) += point;
    return *this;
}



bool CRangeAddress::IsRows() const
{
    return GetLeft() == CAddressItem::MinCol() && GetRight() == CAddressItem::MaxCol();
}

bool CRangeAddress::IsColumns() const
{
    return GetTop() == CAddressItem::MinRow() && GetBottom() == CAddressItem::MaxRow();
}

long CRangeAddress::GetCount() const
{
    return (GetRight() - GetLeft() + 1) * (GetBottom() - GetTop() + 1);
}
