
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#include "stdafx.h"
#include "Application.h"

using namespace AutomateIT;
using namespace AutomateIT::Utils;
using namespace AutomateIT::ExcelClone;
using namespace AutomateIT::OpenOfficeWrapper;

int CApplication::SingletonSection::m_RefCounter = 0;
CApplication::SingletonSection* CApplication::SingletonSection::m_pInstance = NULL; 

CApplication::CApplication() :
m_pInstance(NULL)
{
    TRACINGMETHOD; 
    m_pInstance = SingletonSection::CreateInstance();
    m_pInstance->Initialize(this);
}

CApplication::~CApplication()
{
    TRACINGMETHOD; 
    SingletonSection::FreeInstance();
}

COOApplicationPtr CApplication::GetOOApplication() const
{
    return m_pInstance->GetOOApp();
}

HRESULT CApplication::ResetWorkbooks()
{
    TRACINGMETHOD; 
    hr = m_pInstance->GetWorkbooks()->Reset();
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Application( 
    _Application **RHS)
{
    TRACINGMETHOD;
    (*RHS) = this;
    (*RHS)->AddRef();
     return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Creator( 
    XlCreator *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Parent( 
    Application **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ActiveCell( 
    Range **RHS)
{
    TRACINGMETHOD;

    CWorksheetPtr pWorksheet(NULL);
    hr = get_ActiveSheet((IDispatch**)&pWorksheet);
    CHECKERROR;

    COORangePtr pOORange(NULL);
    hr = pWorksheet->GetParentWorkbook()->GetOODocument()->get_CurrentSelection(&pOORange);
    CHECKERROR;

    CRangeObject* pRangeObject(NULL);
    hr = CRangeObject::CreateInstance(&pRangeObject);
    CHECKERROR;
    CRangePtr pRange(pRangeObject);
    hr = pRange->Initialize(this, pWorksheet, pOORange);
    CHECKERROR;

    (*RHS) = pRange;
    (*RHS)->AddRef();

    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ActiveChart( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ActiveDialog( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ActiveMenuBar( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ActivePrinter( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ActivePrinter( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ActiveSheet( 
    IDispatch **RHS)
{ 
    TRACINGMETHOD; 
    CSheetsPtr pSheetsPtr(NULL);
    hr = get_Sheets((Sheets**)&pSheetsPtr);
    CHECKERROR;
    CWorksheetPtr pWorksheet(NULL);
    hr = pSheetsPtr->GetActiveSheet(&pWorksheet);
    CHECKERROR;

    m_garbageList.AddTail(pWorksheet.p);

    *RHS = pWorksheet;
    (*RHS)->AddRef();
    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ActiveWindow( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ActiveWorkbook( 
    _Workbook **RHS)
{ 
    TRACINGMETHOD; 
    CWorkbookPtr pWorkbook(NULL); 
    hr = m_pInstance->GetWorkbooks()->GetActiveWorkbook(&pWorkbook);
    CHECKERROR;

    (*RHS) = pWorkbook;
    (*RHS)->AddRef();
    return hr; 
}

HRESULT STDMETHODCALLTYPE CApplication::get_AddIns( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Assistant( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Calculate( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Cells( 
    Range **RHS)
{ 
    TRACINGMETHOD; 
    CWorkbookPtr pWorkbookPtr(NULL);
    hr = m_pInstance->GetWorkbooks()->GetActiveWorkbook(&pWorkbookPtr);
    CHECKERROR;
    CSheetsPtr pSheets(NULL);
    pWorkbookPtr->get_Sheets((Sheets **) &pSheets);
    CHECKERROR;
    CWorksheetPtr pWorksheet;
    hr = pSheets->GetActiveSheet(&pWorksheet);
    CHECKERROR;

    // garbage
    m_garbageList.AddTail(pWorksheet.p);

    hr = pWorksheet->get_Cells (RHS);
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Charts( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Columns( 
    Range **RHS)
{
    TRACINGMETHOD;
    CWorkbookPtr pWorkbookPtr(NULL);
    hr = m_pInstance->GetWorkbooks()->GetActiveWorkbook(&pWorkbookPtr);
    CHECKERROR;
    CSheetsPtr pSheets(NULL);
    pWorkbookPtr->get_Sheets((Sheets **) &pSheets);
    CHECKERROR;
    CWorksheetPtr pWorksheet(NULL);
    hr = pSheets->GetActiveSheet(&pWorksheet);
    CHECKERROR;

    m_garbageList.AddTail(pWorksheet.p);

    hr = pWorksheet->get_Columns (RHS);
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CommandBars( 
    IDispatch **RHS)
{ 
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DDEAppReturnCode( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::DDEExecute( 
    long Channel,
    BSTR String,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::DDEInitiate( 
    BSTR App,
    BSTR Topic,
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::DDEPoke( 
    long Channel,
    VARIANT Item,
    VARIANT Data,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::DDERequest( 
    long Channel,
    BSTR Item,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::DDETerminate( 
    long Channel,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DialogSheets( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Evaluate( 
    VARIANT Name,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::_Evaluate( 
    VARIANT Name,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::ExecuteExcel4Macro( 
    BSTR String,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Intersect( 
    Range *Arg1,
    Range *Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT Arg5,
    VARIANT Arg6,
    VARIANT Arg7,
    VARIANT Arg8,
    VARIANT Arg9,
    VARIANT Arg10,
    VARIANT Arg11,
    VARIANT Arg12,
    VARIANT Arg13,
    VARIANT Arg14,
    VARIANT Arg15,
    VARIANT Arg16,
    VARIANT Arg17,
    VARIANT Arg18,
    VARIANT Arg19,
    VARIANT Arg20,
    VARIANT Arg21,
    VARIANT Arg22,
    VARIANT Arg23,
    VARIANT Arg24,
    VARIANT Arg25,
    VARIANT Arg26,
    VARIANT Arg27,
    VARIANT Arg28,
    VARIANT Arg29,
    VARIANT Arg30,
    long lcid,
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MenuBars( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Modules( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Names( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Range( 
    VARIANT Cell1,
    VARIANT Cell2,
    Range **RHS)
{
    TRACINGMETHOD;

    IDispatchPtr pDisp(NULL);
    hr = get_ActiveSheet(&pDisp);
    CHECKERROR;

    m_garbageList.AddTail(pDisp);

    CComPtr<_Worksheet> pWorksheet(NULL);
    hr = pDisp->QueryInterface(&pWorksheet);
    CHECKERROR;
    
    return pWorksheet->get_Range(Cell1, Cell2, RHS);;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Rows( 
    Range **RHS)
{
    TRACINGMETHOD;
    CWorkbookPtr pWorkbookPtr(NULL);
    hr = m_pInstance->GetWorkbooks()->GetActiveWorkbook(&pWorkbookPtr);
    CHECKERROR;
    CSheetsPtr pSheets(NULL);
    pWorkbookPtr->get_Sheets((Sheets **) &pSheets);
    CHECKERROR;
    CWorksheetPtr pWorksheet(NULL);
    hr = pSheets->GetActiveSheet(&pWorksheet);
    CHECKERROR;
    
    m_garbageList.AddTail(pWorksheet.p);

    hr = pWorksheet->get_Rows (RHS);
    CHECKERROR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::Run( 
    VARIANT Macro,
    VARIANT Arg1,
    VARIANT Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT Arg5,
    VARIANT Arg6,
    VARIANT Arg7,
    VARIANT Arg8,
    VARIANT Arg9,
    VARIANT Arg10,
    VARIANT Arg11,
    VARIANT Arg12,
    VARIANT Arg13,
    VARIANT Arg14,
    VARIANT Arg15,
    VARIANT Arg16,
    VARIANT Arg17,
    VARIANT Arg18,
    VARIANT Arg19,
    VARIANT Arg20,
    VARIANT Arg21,
    VARIANT Arg22,
    VARIANT Arg23,
    VARIANT Arg24,
    VARIANT Arg25,
    VARIANT Arg26,
    VARIANT Arg27,
    VARIANT Arg28,
    VARIANT Arg29,
    VARIANT Arg30,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::_Run2( 
    VARIANT Macro,
    VARIANT Arg1,
    VARIANT Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT Arg5,
    VARIANT Arg6,
    VARIANT Arg7,
    VARIANT Arg8,
    VARIANT Arg9,
    VARIANT Arg10,
    VARIANT Arg11,
    VARIANT Arg12,
    VARIANT Arg13,
    VARIANT Arg14,
    VARIANT Arg15,
    VARIANT Arg16,
    VARIANT Arg17,
    VARIANT Arg18,
    VARIANT Arg19,
    VARIANT Arg20,
    VARIANT Arg21,
    VARIANT Arg22,
    VARIANT Arg23,
    VARIANT Arg24,
    VARIANT Arg25,
    VARIANT Arg26,
    VARIANT Arg27,
    VARIANT Arg28,
    VARIANT Arg29,
    VARIANT Arg30,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Selection( 
    long lcid,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    CComPtr<Range> pRange(NULL);
    hr = get_ActiveCell(&pRange);
    CHECKERROR;
    
    (*RHS) = pRange.p;
    (*RHS)->AddRef();

    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::SendKeys( 
    VARIANT Keys,
    VARIANT Wait,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Sheets( 
    Sheets **RHS)
{ 
    TRACINGMETHOD; 
    CWorkbookPtr pWorkbookPtr(NULL);
    hr = m_pInstance->GetWorkbooks()->GetActiveWorkbook(&pWorkbookPtr);
    CHECKERROR;
    return pWorkbookPtr->get_Sheets(RHS);
}

HRESULT STDMETHODCALLTYPE CApplication::get_ShortcutMenus( 
    long Index,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ThisWorkbook( 
    long lcid,
    _Workbook **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Toolbars( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Union( 
    Range *Arg1,
    Range *Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT Arg5,
    VARIANT Arg6,
    VARIANT Arg7,
    VARIANT Arg8,
    VARIANT Arg9,
    VARIANT Arg10,
    VARIANT Arg11,
    VARIANT Arg12,
    VARIANT Arg13,
    VARIANT Arg14,
    VARIANT Arg15,
    VARIANT Arg16,
    VARIANT Arg17,
    VARIANT Arg18,
    VARIANT Arg19,
    VARIANT Arg20,
    VARIANT Arg21,
    VARIANT Arg22,
    VARIANT Arg23,
    VARIANT Arg24,
    VARIANT Arg25,
    VARIANT Arg26,
    VARIANT Arg27,
    VARIANT Arg28,
    VARIANT Arg29,
    VARIANT Arg30,
    long lcid,
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Windows( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Workbooks( 
    Workbooks **RHS)
{ 
    TRACINGMETHOD; 
    *RHS = m_pInstance->GetWorkbooks();
    (*RHS)->AddRef();
    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::get_WorksheetFunction( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Worksheets( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    return get_Sheets(RHS);
}

HRESULT STDMETHODCALLTYPE CApplication::get_Excel4IntlMacroSheets( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Excel4MacroSheets( 
    Sheets **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::ActivateMicrosoftApp( 
    XlMSApplication Index,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::AddChartAutoFormat( 
    VARIANT Chart,
    BSTR Name,
    VARIANT Description,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::AddCustomList( 
    VARIANT ListArray,
    VARIANT ByRow,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_AlertBeforeOverwriting( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_AlertBeforeOverwriting( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_AltStartupPath( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_AltStartupPath( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_AskToUpdateLinks( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_AskToUpdateLinks( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_EnableAnimations( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_EnableAnimations( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_AutoCorrect( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Build( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CalculateBeforeSave( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_CalculateBeforeSave( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Calculation( 
    long lcid,
    XlCalculation *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_Calculation( 
    long lcid,
    XlCalculation RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Caller( 
    VARIANT Index,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CanPlaySounds( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CanRecordSounds( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Caption( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    // stub
    *RHS = ::SysAllocString(OLESTR("Microsoft Excel - Book1"));
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_Caption( 
    BSTR RHS)
{
    TRACINGMETHOD;
    // stub
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CellDragAndDrop( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_CellDragAndDrop( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::CentimetersToPoints( 
    double Centimeters,
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::CheckSpelling( 
    BSTR Word,
    VARIANT CustomDictionary,
    VARIANT IgnoreUppercase,
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ClipboardFormats( 
    VARIANT Index,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayClipboardWindow( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayClipboardWindow( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ColorButtons( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ColorButtons( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CommandUnderlines( 
    long lcid,
    XlCommandUnderlines *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_CommandUnderlines( 
    long lcid,
    XlCommandUnderlines RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ConstrainNumeric( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ConstrainNumeric( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::ConvertFormula( 
    VARIANT Formula,
    XlReferenceStyle FromReferenceStyle,
    VARIANT ToReferenceStyle,
    VARIANT ToAbsolute,
    VARIANT RelativeTo,
    long lcid,
    VARIANT *RHS)
{ 
    TRACINGMETHOD; 
 
    //  format: 
    //  =
    //  R
    //  nRowNumber
    //  C
    //  nColNumber
    std::wstring sFormula = Formula.bstrVal;
    std::wstring sNumber  = L"";
    std::wstring::iterator iFormula     = sFormula.begin();
    std::wstring::iterator iFormulaEnd  = sFormula.end();
    std::wstring sResult = L"=$";
    std::wstring sRow    = L"";
    std::wstring sColumn = L"";

    long    lNumber;
    iFormula += 2;
    while ((iFormula != iFormulaEnd) && (*iFormula <= L'9') && (*iFormula >= L'0'))
    {
        sRow += *iFormula;
        iFormula++;
    }

    if (iFormula != iFormulaEnd) iFormula++;

    while ((iFormula != iFormulaEnd) && (*iFormula <= L'9') && (*iFormula >= L'0'))
    {
        sNumber += *iFormula;
        iFormula++;
    }

    lNumber = _wtoi(sNumber.c_str());



    if (lNumber < 26)
    {
        sColumn += ((WCHAR) lNumber + L'A' - 1);
    } else 
    {
        sColumn += ((WCHAR) (lNumber / 26) + L'A' - 1);
        sColumn += ((WCHAR) (lNumber % 26) + L'A' - 1);
    }

    sResult += (sColumn + L"$" + sRow);

    RHS->bstrVal = ::SysAllocString(sResult.c_str());
    RHS->vt      = VT_BSTR;
    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CopyObjectsWithCells( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_CopyObjectsWithCells( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Cursor( 
    long lcid,
    XlMousePointer *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_Cursor( 
    long lcid,
    XlMousePointer RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CustomListCount( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CutCopyMode( 
    long lcid,
    XlCutCopyMode *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_CutCopyMode( 
    long lcid,
    XlCutCopyMode RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DataEntryMode( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DataEntryMode( 
    long lcid,
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy1( 
    VARIANT Arg1,
    VARIANT Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy2( 
    VARIANT Arg1,
    VARIANT Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT Arg5,
    VARIANT Arg6,
    VARIANT Arg7,
    VARIANT Arg8,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy3( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy4( 
    VARIANT Arg1,
    VARIANT Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT Arg5,
    VARIANT Arg6,
    VARIANT Arg7,
    VARIANT Arg8,
    VARIANT Arg9,
    VARIANT Arg10,
    VARIANT Arg11,
    VARIANT Arg12,
    VARIANT Arg13,
    VARIANT Arg14,
    VARIANT Arg15,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy5( 
    VARIANT Arg1,
    VARIANT Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT Arg5,
    VARIANT Arg6,
    VARIANT Arg7,
    VARIANT Arg8,
    VARIANT Arg9,
    VARIANT Arg10,
    VARIANT Arg11,
    VARIANT Arg12,
    VARIANT Arg13,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy6( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy7( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy8( 
    VARIANT Arg1,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy9( 
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy10( 
    VARIANT arg,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy11( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get__Default( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DefaultFilePath( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DefaultFilePath( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::DeleteChartAutoFormat( 
    BSTR Name,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::DeleteCustomList( 
    long ListNum,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Dialogs( 
    IDispatch **RHS)
{ 
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayAlerts( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    // stub
    (*RHS) = VARIANT_TRUE;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayAlerts( 
    long lcid,
    VARIANT_BOOL RHS)
{ 
    TRACINGMETHOD; 
    // stub
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayFormulaBar( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayFormulaBar( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayFullScreen( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayFullScreen( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayNoteIndicator( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayNoteIndicator( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayCommentIndicator( 
    XlCommentDisplayMode *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayCommentIndicator( 
    XlCommentDisplayMode RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayExcel4Menus( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayExcel4Menus( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayRecentFiles( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayRecentFiles( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayScrollBars( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayScrollBars( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayStatusBar( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayStatusBar( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::DoubleClick( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_EditDirectlyInCell( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_EditDirectlyInCell( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_EnableAutoComplete( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_EnableAutoComplete( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_EnableCancelKey( 
    long lcid,
    XlEnableCancelKey *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_EnableCancelKey( 
    long lcid,
    XlEnableCancelKey RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_EnableSound( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_EnableSound( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_EnableTipWizard( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_EnableTipWizard( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_FileConverters( 
    VARIANT Index1,
    VARIANT Index2,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_FileSearch( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_FileFind( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::_FindFile( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_FixedDecimal( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_FixedDecimal( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_FixedDecimalPlaces( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_FixedDecimalPlaces( 
    long lcid,
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::GetCustomListContents( 
    long ListNum,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::GetCustomListNum( 
    VARIANT ListArray,
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::GetOpenFilename( 
    VARIANT FileFilter,
    VARIANT FilterIndex,
    VARIANT Title,
    VARIANT ButtonText,
    VARIANT MultiSelect,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::GetSaveAsFilename( 
    VARIANT InitialFilename,
    VARIANT FileFilter,
    VARIANT FilterIndex,
    VARIANT Title,
    VARIANT ButtonText,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Goto( 
    VARIANT Reference,
    VARIANT Scroll,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Height( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_Height( 
    long lcid,
    double RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Help( 
    VARIANT HelpFile,
    VARIANT HelpContextID,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_IgnoreRemoteRequests( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_IgnoreRemoteRequests( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::InchesToPoints( 
    double Inches,
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::InputBox( 
    BSTR Prompt,
    VARIANT Title,
    VARIANT Default,
    VARIANT Left,
    VARIANT Top,
    VARIANT HelpFile,
    VARIANT HelpContextID,
    VARIANT Type,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Interactive( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_Interactive( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_International( 
    VARIANT Index,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Iteration( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_Iteration( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_LargeButtons( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_LargeButtons( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Left( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_Left( 
    long lcid,
    double RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_LibraryPath( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::MacroOptions( 
    VARIANT Macro,
    VARIANT Description,
    VARIANT HasMenu,
    VARIANT MenuText,
    VARIANT HasShortcutKey,
    VARIANT ShortcutKey,
    VARIANT Category,
    VARIANT StatusBar,
    VARIANT HelpContextID,
    VARIANT HelpFile,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::MailLogoff( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::MailLogon( 
    VARIANT Name,
    VARIANT Password,
    VARIANT DownloadNewMail,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MailSession( 
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MailSystem( 
    long lcid,
    XlMailSystem *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MathCoprocessorAvailable( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MaxChange( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_MaxChange( 
    long lcid,
    double RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MaxIterations( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_MaxIterations( 
    long lcid,
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MemoryFree( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MemoryTotal( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MemoryUsed( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MouseAvailable( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MoveAfterReturn( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_MoveAfterReturn( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MoveAfterReturnDirection( 
    long lcid,
    XlDirection *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_MoveAfterReturnDirection( 
    long lcid,
    XlDirection RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_RecentFiles( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Name( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::NextLetter( 
    long lcid,
    _Workbook **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_NetworkTemplatesPath( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ODBCErrors( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ODBCTimeout( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ODBCTimeout( 
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OnCalculate( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_OnCalculate( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OnData( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_OnData( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OnDoubleClick( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_OnDoubleClick( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OnEntry( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_OnEntry( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::OnKey( 
    BSTR Key,
    VARIANT Procedure,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::OnRepeat( 
    BSTR Text,
    BSTR Procedure,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OnSheetActivate( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_OnSheetActivate( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OnSheetDeactivate( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_OnSheetDeactivate( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::OnTime( 
    VARIANT EarliestTime,
    BSTR Procedure,
    VARIANT LatestTime,
    VARIANT Schedule,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::OnUndo( 
    BSTR Text,
    BSTR Procedure,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OnWindow( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_OnWindow( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OperatingSystem( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OrganizationName( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Path( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_PathSeparator( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_PreviousSelections( 
    VARIANT Index,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_PivotTableSelection( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_PivotTableSelection( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_PromptForSummaryInfo( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_PromptForSummaryInfo( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Quit( void)
{ 
    TRACINGMETHOD; 

    CWorkbookPtr activeWB(NULL);
    hr = m_pInstance->GetWorkbooks()->GetActiveWorkbook(&activeWB);
    if (!FAILED(hr))
    {
        hr = activeWB->Close(CComVariant(false), CComVariant(), CComVariant(), 0);
        CHECKERROR;
    }
    hr = S_OK;
    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::RecordMacro( 
    VARIANT BasicCode,
    VARIANT XlmCode,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_RecordRelative( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ReferenceStyle( 
    long lcid,
    XlReferenceStyle *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ReferenceStyle( 
    long lcid,
    XlReferenceStyle RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_RegisteredFunctions( 
    VARIANT Index1,
    VARIANT Index2,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::RegisterXLL( 
    BSTR Filename,
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Repeat( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::ResetTipWizard( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_RollZoom( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_RollZoom( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Save( 
    VARIANT Filename,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::SaveWorkspace( 
    VARIANT Filename,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ScreenUpdating( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    // stub
    *RHS = VARIANT_TRUE;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ScreenUpdating( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    // stub
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::SetDefaultChart( 
    VARIANT FormatName,
    VARIANT Gallery)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_SheetsInNewWorkbook( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_SheetsInNewWorkbook( 
    long lcid,
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ShowChartTipNames( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ShowChartTipNames( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ShowChartTipValues( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ShowChartTipValues( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_StandardFont( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_StandardFont( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_StandardFontSize( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_StandardFontSize( 
    long lcid,
    double RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_StartupPath( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_StatusBar( 
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_StatusBar( 
    long lcid,
    VARIANT RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_TemplatesPath( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ShowToolTips( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ShowToolTips( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Top( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_Top( 
    long lcid,
    double RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DefaultSaveFormat( 
    XlFileFormat *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DefaultSaveFormat( 
    XlFileFormat RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_TransitionMenuKey( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_TransitionMenuKey( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_TransitionMenuKeyAction( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_TransitionMenuKeyAction( 
    long lcid,
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_TransitionNavigKeys( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_TransitionNavigKeys( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Undo( 
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_UsableHeight( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_UsableWidth( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_UserControl( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    // stub
    *RHS = VARIANT_TRUE;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_UserControl( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    // stub
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_UserName( 
    long lcid,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_UserName( 
    long lcid,
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Value( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_VBE( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Version( 
    long lcid,
    BSTR *RHS)
{ 
    TRACINGMETHOD; 

    *RHS = ::SysAllocString(OLESTR("11.0"));

    return hr;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Visible( 
    long lcid,
    VARIANT_BOOL *RHS)
{ 
    TRACINGMETHOD; 
    //CWorkbookPtr pWorkbook(NULL); 
    //hr = m_pInstance->GetWorkbooks()->GetActiveWorkbook(&pWorkbook);
    //CHECKERROR;

    //bool visible;
    //hr = pWorkbook->GetOODocument()->GetDocument()->GetVisible(&visible);
    //CHECKERROR;
    //(*RHS) = visible ? VARIANT_TRUE : VARIANT_FALSE;

    // hr = m_pInstance->GetOOApp()->GetVisible(RHS);
    // CHECKERROR;
    (*RHS) = VARIANT_TRUE;
    return S_OK; 
}

HRESULT STDMETHODCALLTYPE CApplication::put_Visible( 
    long lcid,
    VARIANT_BOOL RHS)
{ 
    TRACINGMETHOD; 
    long count(0);
    hr = m_pInstance->GetWorkbooks()->get_Count(&count);
    CHECKERROR;
    if (count > 0)
    {
        CWorkbookPtr pWorkbook(NULL); 
        hr = m_pInstance->GetWorkbooks()->GetActiveWorkbook(&pWorkbook);
        CHECKERROR;

        hr = pWorkbook->GetOODocument()->GetDocument()->SetVisible(RHS != VARIANT_FALSE);
        CHECKERROR;

        //hr = m_pInstance->GetOOApp()->SetVisible(RHS);
        //CHECKERROR;
        return hr; 
    }
    else
    {
        // do nothing
        return hr;
    }
}

HRESULT STDMETHODCALLTYPE CApplication::Volatile( 
    VARIANT Volatile,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::_Wait( 
    VARIANT Time,
    long lcid)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Width( 
    long lcid,
    double *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_Width( 
    long lcid,
    double RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_WindowsForPens( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_WindowState( 
    long lcid,
    XlWindowState *RHS)
{
    TRACINGMETHOD;
    // stub
    *RHS = xlMaximized;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_WindowState( 
    long lcid,
    XlWindowState RHS)
{
    TRACINGMETHOD;
    // stub
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_UILanguage( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_UILanguage( 
    long lcid,
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DefaultSheetDirection( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DefaultSheetDirection( 
    long lcid,
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CursorMovement( 
    long lcid,
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_CursorMovement( 
    long lcid,
    long RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ControlCharacters( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ControlCharacters( 
    long lcid,
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::_WSFunction( 
    VARIANT Arg1,
    VARIANT Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT Arg5,
    VARIANT Arg6,
    VARIANT Arg7,
    VARIANT Arg8,
    VARIANT Arg9,
    VARIANT Arg10,
    VARIANT Arg11,
    VARIANT Arg12,
    VARIANT Arg13,
    VARIANT Arg14,
    VARIANT Arg15,
    VARIANT Arg16,
    VARIANT Arg17,
    VARIANT Arg18,
    VARIANT Arg19,
    VARIANT Arg20,
    VARIANT Arg21,
    VARIANT Arg22,
    VARIANT Arg23,
    VARIANT Arg24,
    VARIANT Arg25,
    VARIANT Arg26,
    VARIANT Arg27,
    VARIANT Arg28,
    VARIANT Arg29,
    VARIANT Arg30,
    long lcid,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_EnableEvents( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    // stub
    *RHS = VARIANT_TRUE;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_EnableEvents( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    // stub
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayInfoWindow( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayInfoWindow( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Wait( 
    VARIANT Time,
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ExtendList( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ExtendList( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_OLEDBErrors( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::GetPhonetic( 
    VARIANT Text,
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_COMAddIns( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DefaultWebOptions( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ProductCode( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_UserLibraryPath( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_AutoPercentEntry( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_AutoPercentEntry( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_LanguageSettings( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Dummy101( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy12( 
    IDispatch *p1,
    IDispatch *p2)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_AnswerWizard( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::CalculateFull( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::FindFile( 
    long lcid,
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CalculationVersion( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ShowWindowsInTaskbar( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ShowWindowsInTaskbar( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_FeatureInstall( 
    MsoFeatureInstall *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_FeatureInstall( 
    MsoFeatureInstall RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Ready( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy13( 
    VARIANT Arg1,
    VARIANT Arg2,
    VARIANT Arg3,
    VARIANT Arg4,
    VARIANT Arg5,
    VARIANT Arg6,
    VARIANT Arg7,
    VARIANT Arg8,
    VARIANT Arg9,
    VARIANT Arg10,
    VARIANT Arg11,
    VARIANT Arg12,
    VARIANT Arg13,
    VARIANT Arg14,
    VARIANT Arg15,
    VARIANT Arg16,
    VARIANT Arg17,
    VARIANT Arg18,
    VARIANT Arg19,
    VARIANT Arg20,
    VARIANT Arg21,
    VARIANT Arg22,
    VARIANT Arg23,
    VARIANT Arg24,
    VARIANT Arg25,
    VARIANT Arg26,
    VARIANT Arg27,
    VARIANT Arg28,
    VARIANT Arg29,
    VARIANT Arg30,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_FindFormat( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::putref_FindFormat( 
    IDispatch *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ReplaceFormat( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::putref_ReplaceFormat( 
    IDispatch *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_UsedObjects( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CalculationState( 
    XlCalculationState *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_CalculationInterruptKey( 
    XlCalculationInterruptKey *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_CalculationInterruptKey( 
    XlCalculationInterruptKey RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Watches( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayFunctionToolTips( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayFunctionToolTips( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_AutomationSecurity( 
    MsoAutomationSecurity *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_AutomationSecurity( 
    MsoAutomationSecurity RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_FileDialog( 
    MsoFileDialogType fileDialogType,
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Dummy14( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::CalculateFullRebuild( void)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayPasteOptions( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayPasteOptions( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayInsertOptions( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayInsertOptions( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_GenerateGetPivotData( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_GenerateGetPivotData( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_AutoRecover( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Hwnd( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Hinstance( 
    long *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::CheckAbort( 
    VARIANT KeepAbort)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ErrorCheckingOptions( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_AutoFormatAsYouTypeReplaceHyperlinks( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_AutoFormatAsYouTypeReplaceHyperlinks( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_SmartTagRecognizers( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_NewWorkbook( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_SpellingOptions( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_Speech( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_MapPaperSize( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_MapPaperSize( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ShowStartupDialog( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ShowStartupDialog( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DecimalSeparator( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DecimalSeparator( 
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ThousandsSeparator( 
    BSTR *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_ThousandsSeparator( 
    BSTR RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_UseSystemSeparators( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_UseSystemSeparators( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ThisCell( 
    Range **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_RTD( 
    IDispatch **RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_DisplayDocumentActionTaskPane( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::put_DisplayDocumentActionTaskPane( 
    VARIANT_BOOL RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::DisplayXMLSourcePane( 
    VARIANT XmlMap)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::get_ArbitraryXMLSupportAvailable( 
    VARIANT_BOOL *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}

HRESULT STDMETHODCALLTYPE CApplication::Support( 
    IDispatch *Object,
    long ID,
    VARIANT arg,
    VARIANT *RHS)
{
    TRACINGMETHOD;
    TRACINGNOTIMPL;
    return S_OK;
}


CApplication::SingletonSection* CApplication::SingletonSection::CreateInstance()
{
    TRACINGMETHOD; 
    if (m_RefCounter == 0)
    {
        m_pInstance = new SingletonSection();
    }
    m_RefCounter++;
    return m_pInstance;
}

void CApplication::SingletonSection::FreeInstance()
{
    TRACINGMETHOD; 
    m_RefCounter--;
    if (m_RefCounter == 0)
    {
        delete m_pInstance;
        m_pInstance = NULL;
    }
}

OpenOfficeWrapper::COOApplicationPtr CApplication::SingletonSection::GetOOApp()
{
    return m_pOOApp;
}

CWorkbooksPtr CApplication::SingletonSection::GetWorkbooks()
{
    return m_Workbooks;
}

CApplication::SingletonSection::SingletonSection() 
: m_pOOApp(NULL), m_Workbooks(NULL)
{
    TRACINGMETHOD; 
}

CApplication::SingletonSection::~SingletonSection()
{
    TRACINGMETHOD; 
}

void CApplication::SingletonSection::Initialize( CApplication* pApplication )
{
    TRACINGMETHOD; 
    if(m_RefCounter == 1)
    {
        COOApplicationObject* pOOApp(NULL);
        hr = COOApplicationObject::CreateInstance(&pOOApp);
        CHECKERROR_;

        m_pOOApp = pOOApp;


        CWorkbooksObject* pWBs(NULL);
        hr = CWorkbooksObject::CreateInstance(&pWBs);
        CHECKERROR_;

        m_Workbooks = pWBs;
        m_Workbooks->Initialize(pApplication);
    }
}

