
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _SHEETS_H_
#define _SHEETS_H_

#include "ExcelClone.h"
#include "DispatchHook.h"
#include "Worksheet.h"
#include "OOSheets.h"

/*
CSheets class provides functionality of the Sheets objects by using OpenOffice. 
It implements Sheets COM interface.
*/

namespace AutomateIT
{
	namespace ExcelClone
	{
		class CApplication;
		class CWorkbook;
		class CWorksheet;

		class CSheets :
			public CComObjectRootEx<CComSingleThreadModel>,
			public CDispatchHook< IDispatchImpl<Sheets, &IID_Sheets, &LIBID_ExcelCloneLib, 1, 0> >
		{
		public:
			CSheets();
			virtual ~CSheets();

			BEGIN_COM_MAP(CSheets)
				COM_INTERFACE_ENTRY(Sheets)
				COM_INTERFACE_ENTRY(IDispatch)
			END_COM_MAP()

		public:



            HRESULT STDMETHODCALLTYPE get_Application( 
                 _Application **RHS);

            HRESULT STDMETHODCALLTYPE get_Creator( 
                 XlCreator *RHS);

            HRESULT STDMETHODCALLTYPE get_Parent( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Add( 
                 VARIANT Before,
                 VARIANT After,
                 VARIANT Count,
                 VARIANT Type,
                 long lcid,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Copy( 
                 VARIANT Before,
                 VARIANT After,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Count( 
                 long *RHS);

            HRESULT STDMETHODCALLTYPE Delete( 
                 long lcid);

            HRESULT STDMETHODCALLTYPE FillAcrossSheets( 
                 Range *Range,
                 XlFillWith Type,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_Item( 
                 VARIANT Index,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE Move( 
                 VARIANT Before,
                 VARIANT After,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get__NewEnum( 
                 IUnknown **RHS);

            HRESULT STDMETHODCALLTYPE _PrintOut( 
                 VARIANT From,
                 VARIANT To,
                 VARIANT Copies,
                 VARIANT Preview,
                 VARIANT ActivePrinter,
                 VARIANT PrintToFile,
                 VARIANT Collate,
                 long lcid);

            HRESULT STDMETHODCALLTYPE PrintPreview( 
                 VARIANT EnableChanges,
                 long lcid);

            HRESULT STDMETHODCALLTYPE Select( 
                 VARIANT Replace,
                 long lcid);

            HRESULT STDMETHODCALLTYPE get_HPageBreaks( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_VPageBreaks( 
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE get_Visible( 
                 long lcid,
                 VARIANT *RHS);

            HRESULT STDMETHODCALLTYPE put_Visible( 
                 long lcid,
                 VARIANT RHS);

            HRESULT STDMETHODCALLTYPE get__Default( 
                 VARIANT Index,
                 IDispatch **RHS);

            HRESULT STDMETHODCALLTYPE PrintOut( 
                 VARIANT From,
                 VARIANT To,
                 VARIANT Copies,
                 VARIANT Preview,
                 VARIANT ActivePrinter,
                 VARIANT PrintToFile,
                 VARIANT Collate,
                 VARIANT PrToFileName,
                 long lcid);

			
		public:
			
            STDMETHOD(Initialize)(CApplication* pParentApplication, CWorkbook *pWorkbook);
			STDMETHOD(GetActiveSheet)(CWorksheet** worksheet);

            OpenOfficeWrapper::COOSheetsPtr GetOOSheetObject();

		protected:
            OpenOfficeWrapper::COOSheetsPtr m_pSheets;
            CWorkbook* m_ParentWorkbook;
			CApplication* m_ParentApplication;

            OpenOfficeWrapper::DispatchList m_garbageList;
		};

		typedef CComObject<CSheets> CSheetsObject;
        typedef CComPtr<CSheets> CSheetsPtr;

	}
}

#endif  //_SHEETS_H_
