/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/


#ifndef __NUMBERING_TYPE_ENUM_H_
#define __NUMBERING_TYPE_ENUM_H_

namespace AutomateIT
{
    namespace OpenOfficeWrapper
    {
        namespace OpenOfficeNumberingType
        {
            typedef enum NumberingType
            {
                CHARS_UPPER_LETTER = 0,                // Numbering is put in upper case letters as "A, B, C, D, ...".
                CHARS_LOWER_LETTER = 1,                // Numbering is in lower case letters as "a, b, c, e,...".
                ROMAN_UPPER = 2,                       // Numbering is in Roman numbers with upper case letters as "I, II, III, IV, ...".
                ROMAN_LOWER	= 3,                       // Numbering is in Roman numbers with lower case letters as "i, ii, iii, iv, ...".
                ARABIC = 4,                            // Numbering is in Arabic numbers as "1, 2, 3, 4, ...".  	
                NUMBER_NONE = 5,                       // Numbering is invisible.
                CHAR_SPECIAL = 6,                      // Use a character from a specified font.
                PAGE_DESCRIPTOR = 7,                   // Numbering is specified in the page style.
                BITMAP = 8,                            // Numbering is displayed as a bitmap graphic.	
                CHARS_UPPER_LETTER_N = 9,              // Numbering is put in upper case letters as "A, B, ..., Y, Z, AA, BB, CC, ... AAA, ...".  	
                CHARS_LOWER_LETTER_N = 10,             // Numbering is put in lower case letters as "a, b, ..., y, z, aa, bb, cc, ... aaa, ...".  	
                TRANSLITERATION	= 11,                  // A transliteration module will be used to produce numbers in chinese, japanese, etc.  	
                NATIVE_NUMBERING = 12,                 // The native number supplier service will be called to produce numbers in native languages.  	
                FULLWIDTH_ARABIC = 13,                 // Numbering for full width Arabic number  	
                CIRCLE_NUMBER = 14,                    // Bullet for Circle Number
                NUMBER_LOWER_ZH = 15,                  // Numbering for Chinese lower case number
                NUMBER_UPPER_ZH = 16,                  // Numbering for Chinese upper case number
                NUMBER_UPPER_ZH_TW = 17,               // Numbering for Traditional Chinese upper case number
                TIAN_GAN_ZH = 18,                      // Bullet for Chinese Tian Gan
                DI_ZI_ZH = 19,                         // Bullet for Chinese Di Zi
                NUMBER_TRADITIONAL_JA = 20,            // Numbering for Japanese traditional number
                AIU_FULLWIDTH_JA = 21,                 // Bullet for Japanese AIU fullwidth
                AIU_HALFWIDTH_JA = 22,                 // Bullet for Japanese AIU halfwidth
                IROHA_FULLWIDTH_JA = 23,               // Bullet for Japanese IROHA fullwidth
                IROHA_HALFWIDTH_JA = 24,               // Bullet for Japanese IROHA halfwidth
                NUMBER_UPPER_KO	= 25,                  // Numbering for Korean upper case number
                NUMBER_HANGUL_KO = 26,                 // Numbering for Korean hangul number
                HANGUL_JAMO_KO = 27,                   // Bullet for Korean Hangul Jamo
                HANGUL_SYLLABLE_KO = 28,               // Bullet for Korean Hangul Syllable
                HANGUL_CIRCLED_JAMO_KO = 29,           // Bullet for Korean Hangul Circled Jamo
                HANGUL_CIRCLED_SYLLABLE_KO = 30,       // Bullet for Korean Hangul Circled Syllable  	
                CHARS_ARABIC = 31,                     // Numbering in Arabic alphabet letters  	
                CHARS_THAI = 32,                       // Numbering in Thai alphabet letters
                CHARS_HEBREW = 33,                     // Numbering in Hebrew alphabet letters
                CHARS_NEPALI = 34,                     // Numbering in Nepali alphabet letters
                CHARS_KHMER	= 35,                      // Numbering in Khmer alphabet letters
                CHARS_LAO = 36,                        // Numbering in Lao alphabet letters  	
                CHARS_TIBETAN = 37,                    // Numbering in Tibetan/Dzongkha alphabet letters  	
                CHARS_CYRILLIC_UPPER_LETTER_BG = 38,   // Numbering in Cyrillic alphabet upper case letters as ", , , , ..., , , , , , ... , ".  	
                CHARS_CYRILLIC_LOWER_LETTER_BG = 39,   // Numbering in Cyrillic alphabet lower case letters as ", , , , ..., , ,  , , , ...   ,  ".  	
                CHARS_CYRILLIC_UPPER_LETTER_N_BG = 40, // Numbering in Cyrillic alphabet upper case letters as ", , ..., , , , , , ... , ...".  	
                CHARS_CYRILLIC_LOWER_LETTER_N_BG = 41, // Numbering in Cyrillic alphabet upper case letters as ", , ..., , , , , , ... , ...".  	
                CHARS_CYRILLIC_UPPER_LETTER_RU = 42,   // Numbering in Russian Cyrillic alphabet upper case letters as ", , , , ..., , , , , , ... , ".  	
                CHARS_CYRILLIC_LOWER_LETTER_RU = 43,   // Numbering in Russian Cyrillic alphabet lower case letters as ", , , , ..., , , , , , ... , ".  	
                CHARS_CYRILLIC_UPPER_LETTER_N_RU = 44, // Numbering in Russian Cyrillic alphabet upper case letters as ", , ..., , , , , , ... , ...".  	
                CHARS_CYRILLIC_LOWER_LETTER_N_RU = 45, // Numbering in Russian Cyrillic alphabet upper case letters as ", , ..., , , , , , ... , ...".  	
                CHARS_PERSIAN = 46                     // Numbering in Persian alphabet letters (aa, be, pe, te, ...)  
            };
        }
    }
}

#endif //__NUMBERING_TYPE_ENUM_H_