
/***************************************************************************
*
* AutomateIT -- MS Office COM objects cloner
* 
* Copyright (C) 2005 by ApriorIT <support@apriorit.com>
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU Lesser General Public License as published by
* the Free Software Foundation; either version 2.1 of the License, or
* (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU Lesser General Public License for more details.
* 
* You should have received a copy of the GNU Lesser General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
****************************************************************************/

#ifndef _WORD_DOCUMENT_H_
#define _WORD_DOCUMENT_H_

#include <atlcom.h>
#include "resource.h"       
#include "WordClone.h"
#include "WordStylesCollection.h"
#include "WordVariables.h"
#include "WordListTemplates.h"
#include "WordRange.h"
#include "WordInlineShapes.h"
#include "WordTablesOfContents.h"
#include "WordSections.h"
#include "WordTables.h"

namespace AutomateIT
{
	namespace WordClone
	{
        class CWordDocumentsCollection;

		class ATL_NO_VTABLE CWordDocument : 
			public CComObjectRootEx<CComSingleThreadModel>,
			public CComCoClass<CWordDocument, &CLSID_Document>,
			public IDispatchImpl<_Document, &__uuidof(_Document), &LIBID_WordCloneLib, 8, 3>,
			public IConnectionPointContainer
		{
		public:
			CWordDocument();
			virtual ~CWordDocument();

			//DECLARE_REGISTRY_RESOURCEID(IDR_WORDDOCUMENT)

			BEGIN_COM_MAP(CWordDocument)
				COM_INTERFACE_ENTRY(_Document)
				COM_INTERFACE_ENTRY(IConnectionPointContainer)
			END_COM_MAP()

			DECLARE_PROTECT_FINAL_CONSTRUCT()

			HRESULT FinalConstruct()
			{
				return S_OK;
			}

			void FinalRelease() 
			{
			}

		public:

            // _Document interface methods
			STDMETHOD(get_Name)(BSTR* prop);
			STDMETHOD(get_Application)(IApplication** prop);
			STDMETHOD(get_Creator)(long* prop);
			STDMETHOD(get_Parent)(LPDISPATCH* prop);
			STDMETHOD(get_BuiltInDocumentProperties)(LPDISPATCH* prop);
			STDMETHOD(get_CustomDocumentProperties)(LPDISPATCH* prop);
			STDMETHOD(get_Path)(BSTR* prop);
			STDMETHOD(get_Bookmarks)(IDispatch** prop);
			STDMETHOD(get_Tables)(IDispatch** prop);
			STDMETHOD(get_Footnotes)(IDispatch** prop);
			STDMETHOD(get_Endnotes)(IDispatch** prop);
			STDMETHOD(get_Comments)(IDispatch** prop);
			STDMETHOD(get_Type)(WdDocumentType* prop);
			STDMETHOD(get_AutoHyphenation)(VARIANT_BOOL* prop);
			STDMETHOD(put_AutoHyphenation)(VARIANT_BOOL prop);
			STDMETHOD(get_HyphenateCaps)(VARIANT_BOOL* prop);
			STDMETHOD(put_HyphenateCaps)(VARIANT_BOOL prop);
			STDMETHOD(get_HyphenationZone)(long* prop);
			STDMETHOD(put_HyphenationZone)(long prop);
			STDMETHOD(get_ConsecutiveHyphensLimit)(long* prop);
			STDMETHOD(put_ConsecutiveHyphensLimit)(long prop);
			STDMETHOD(get_Sections)(Sections** prop);
			STDMETHOD(get_Paragraphs)(Paragraphs** prop);
			STDMETHOD(get_Words)(IDispatch** prop);
			STDMETHOD(get_Sentences)(IDispatch** prop);
			STDMETHOD(get_Characters)(IDispatch** prop);
			STDMETHOD(get_Fields)(IDispatch** prop);
			STDMETHOD(get_FormFields)(IDispatch** prop);
			STDMETHOD(get_Styles)(Styles** prop);
			STDMETHOD(get_Frames)(IDispatch** prop);
			STDMETHOD(get_TablesOfFigures)(IDispatch** prop);
			STDMETHOD(get_Variables)(Variables** prop);
			STDMETHOD(get_MailMerge)(IDispatch** prop);
			STDMETHOD(get_Envelope)(IDispatch** prop);
			STDMETHOD(get_FullName)(BSTR* prop);
			STDMETHOD(get_Revisions)(IDispatch** prop);
			STDMETHOD(get_TablesOfContents)(TablesOfContents** prop);
			STDMETHOD(get_TablesOfAuthorities)(IDispatch** prop);
			STDMETHOD(get_PageSetup)(IDispatch** prop);
			STDMETHOD(put_PageSetup)(IDispatch* prop);
			STDMETHOD(get_Windows)(IDispatch** prop);
			STDMETHOD(get_HasRoutingSlip)(VARIANT_BOOL* prop);
			STDMETHOD(put_HasRoutingSlip)(VARIANT_BOOL prop);
			STDMETHOD(get_RoutingSlip)(IDispatch** prop);
			STDMETHOD(get_Routed)(VARIANT_BOOL* prop);
			STDMETHOD(get_TablesOfAuthoritiesCategories)(IDispatch** prop);
			STDMETHOD(get_Indexes)(IDispatch** prop);
			STDMETHOD(get_Saved)(VARIANT_BOOL* prop);
			STDMETHOD(put_Saved)(VARIANT_BOOL prop);
			STDMETHOD(get_Content)(IDispatch** prop);
			STDMETHOD(get_ActiveWindow)(IDispatch** prop);
			STDMETHOD(get_Kind)(WdDocumentKind* prop);
			STDMETHOD(put_Kind)(WdDocumentKind prop);
			STDMETHOD(get_ReadOnly)(VARIANT_BOOL* prop);
			STDMETHOD(get_Subdocuments)(IDispatch** prop);
			STDMETHOD(get_IsMasterDocument)(VARIANT_BOOL* prop);
			STDMETHOD(get_DefaultTabStop)(float* prop);
			STDMETHOD(put_DefaultTabStop)(float prop);
			STDMETHOD(get_EmbedTrueTypeFonts)(VARIANT_BOOL* prop);
			STDMETHOD(put_EmbedTrueTypeFonts)(VARIANT_BOOL prop);
			STDMETHOD(get_SaveFormsData)(VARIANT_BOOL* prop);
			STDMETHOD(put_SaveFormsData)(VARIANT_BOOL prop);
			STDMETHOD(get_ReadOnlyRecommended)(VARIANT_BOOL* prop);
			STDMETHOD(put_ReadOnlyRecommended)(VARIANT_BOOL prop);
			STDMETHOD(get_SaveSubsetFonts)(VARIANT_BOOL* prop);
			STDMETHOD(put_SaveSubsetFonts)(VARIANT_BOOL prop);
			STDMETHOD(get_Compatibility)(WdCompatibility Type, VARIANT_BOOL* prop);
			STDMETHOD(put_Compatibility)(WdCompatibility Type, VARIANT_BOOL prop);
			STDMETHOD(get_StoryRanges)(IDispatch** prop);
			STDMETHOD(get_CommandBars)(IDispatch** prop);
			STDMETHOD(get_IsSubdocument)(VARIANT_BOOL* prop);
			STDMETHOD(get_SaveFormat)(long* prop);
			STDMETHOD(get_ProtectionType)(WdProtectionType* prop);
			STDMETHOD(get_Hyperlinks)(IDispatch** prop);
			STDMETHOD(get_Shapes)(IDispatch** prop);
			STDMETHOD(get_ListTemplates)(ListTemplates** prop);
			STDMETHOD(get_Lists)(IDispatch** prop);
			STDMETHOD(get_UpdateStylesOnOpen)(VARIANT_BOOL* prop);
			STDMETHOD(put_UpdateStylesOnOpen)(VARIANT_BOOL prop);
			STDMETHOD(get_AttachedTemplate)(VARIANT* prop);
			STDMETHOD(put_AttachedTemplate)(VARIANT* prop);
			STDMETHOD(get_InlineShapes)(InlineShapes** prop);
			STDMETHOD(get_Background)(IDispatch** prop);
			STDMETHOD(put_Background)(IDispatch* prop);
			STDMETHOD(get_GrammarChecked)(VARIANT_BOOL* prop);
			STDMETHOD(put_GrammarChecked)(VARIANT_BOOL prop);
			STDMETHOD(get_SpellingChecked)(VARIANT_BOOL* prop);
			STDMETHOD(put_SpellingChecked)(VARIANT_BOOL prop);
			STDMETHOD(get_ShowGrammaticalErrors)(VARIANT_BOOL* prop);
			STDMETHOD(put_ShowGrammaticalErrors)(VARIANT_BOOL prop);
			STDMETHOD(get_ShowSpellingErrors)(VARIANT_BOOL* prop);
			STDMETHOD(put_ShowSpellingErrors)(VARIANT_BOOL prop);
			STDMETHOD(get_Versions)(IDispatch** prop);
			STDMETHOD(get_ShowSummary)(VARIANT_BOOL* prop);
			STDMETHOD(put_ShowSummary)(VARIANT_BOOL prop);
			STDMETHOD(get_SummaryViewMode)(WdSummaryMode* prop);
			STDMETHOD(put_SummaryViewMode)(WdSummaryMode prop);
			STDMETHOD(get_SummaryLength)(long* prop);
			STDMETHOD(put_SummaryLength)(long prop);
			STDMETHOD(get_PrintFractionalWidths)(VARIANT_BOOL* prop);
			STDMETHOD(put_PrintFractionalWidths)(VARIANT_BOOL prop);
			STDMETHOD(get_PrintPostScriptOverText)(VARIANT_BOOL* prop);
			STDMETHOD(put_PrintPostScriptOverText)(VARIANT_BOOL prop);
			STDMETHOD(get_Container)(LPDISPATCH* prop);
			STDMETHOD(get_PrintFormsData)(VARIANT_BOOL* prop);
			STDMETHOD(put_PrintFormsData)(VARIANT_BOOL prop);
			STDMETHOD(get_ListParagraphs)(IDispatch** prop);
			STDMETHOD(put_Password)(BSTR);
			STDMETHOD(put_WritePassword)(BSTR);
			STDMETHOD(get_HasPassword)(VARIANT_BOOL* prop);
			STDMETHOD(get_WriteReserved)(VARIANT_BOOL* prop);
			STDMETHOD(get_ActiveWritingStyle)(VARIANT* LanguageID, BSTR* prop);
			STDMETHOD(put_ActiveWritingStyle)(VARIANT* LanguageID, BSTR prop);
			STDMETHOD(get_UserControl)(VARIANT_BOOL* prop);
			STDMETHOD(put_UserControl)(VARIANT_BOOL prop);
			STDMETHOD(get_HasMailer)(VARIANT_BOOL* prop);
			STDMETHOD(put_HasMailer)(VARIANT_BOOL prop);
			STDMETHOD(get_Mailer)(IDispatch** prop);
			STDMETHOD(get_ReadabilityStatistics)(IDispatch** prop);
			STDMETHOD(get_GrammaticalErrors)(IDispatch** prop);
			STDMETHOD(get_SpellingErrors)(IDispatch** prop);
			STDMETHOD(get_VBProject)(IDispatch** prop);
			STDMETHOD(get_FormsDesign)(VARIANT_BOOL* prop);
			STDMETHOD(get__CodeName)(BSTR* prop);
			STDMETHOD(put__CodeName)(BSTR prop);
			STDMETHOD(get_CodeName)(BSTR* prop);
			STDMETHOD(get_SnapToGrid)(VARIANT_BOOL* prop);
			STDMETHOD(put_SnapToGrid)(VARIANT_BOOL prop);
			STDMETHOD(get_SnapToShapes)(VARIANT_BOOL* prop);
			STDMETHOD(put_SnapToShapes)(VARIANT_BOOL prop);
			STDMETHOD(get_GridDistanceHorizontal)(float* prop);
			STDMETHOD(put_GridDistanceHorizontal)(float prop);
			STDMETHOD(get_GridDistanceVertical)(float* prop);
			STDMETHOD(put_GridDistanceVertical)(float prop);
			STDMETHOD(get_GridOriginHorizontal)(float* prop);
			STDMETHOD(put_GridOriginHorizontal)(float prop);
			STDMETHOD(get_GridOriginVertical)(float* prop);
			STDMETHOD(put_GridOriginVertical)(float prop);
			STDMETHOD(get_GridSpaceBetweenHorizontalLines)(long* prop);
			STDMETHOD(put_GridSpaceBetweenHorizontalLines)(long prop);
			STDMETHOD(get_GridSpaceBetweenVerticalLines)(long* prop);
			STDMETHOD(put_GridSpaceBetweenVerticalLines)(long prop);
			STDMETHOD(get_GridOriginFromMargin)(VARIANT_BOOL* prop);
			STDMETHOD(put_GridOriginFromMargin)(VARIANT_BOOL prop);
			STDMETHOD(get_KerningByAlgorithm)(VARIANT_BOOL* prop);
			STDMETHOD(put_KerningByAlgorithm)(VARIANT_BOOL prop);
			STDMETHOD(get_JustificationMode)(WdJustificationMode* prop);
			STDMETHOD(put_JustificationMode)(WdJustificationMode prop);
			STDMETHOD(get_FarEastLineBreakLevel)(WdFarEastLineBreakLevel* prop);
			STDMETHOD(put_FarEastLineBreakLevel)(WdFarEastLineBreakLevel prop);
			STDMETHOD(get_NoLineBreakBefore)(BSTR* prop);
			STDMETHOD(put_NoLineBreakBefore)(BSTR prop);
			STDMETHOD(get_NoLineBreakAfter)(BSTR* prop);
			STDMETHOD(put_NoLineBreakAfter)(BSTR prop);
			STDMETHOD(get_TrackRevisions)(VARIANT_BOOL* prop);
			STDMETHOD(put_TrackRevisions)(VARIANT_BOOL prop);
			STDMETHOD(get_PrintRevisions)(VARIANT_BOOL* prop);
			STDMETHOD(put_PrintRevisions)(VARIANT_BOOL prop);
			STDMETHOD(get_ShowRevisions)(VARIANT_BOOL* prop);
			STDMETHOD(put_ShowRevisions)(VARIANT_BOOL prop);
			STDMETHOD(Close)(VARIANT* SaveChanges, VARIANT* OriginalFormat, VARIANT* RouteDocument);
			STDMETHOD(SaveAs2000)(VARIANT* FileName, VARIANT* FileFormat, VARIANT* LockComments, VARIANT* Password, VARIANT* AddToRecentFiles, VARIANT* WritePassword, VARIANT* ReadOnlyRecommended, VARIANT* EmbedTrueTypeFonts, VARIANT* SaveNativePictureFormat, VARIANT* SaveFormsData, VARIANT* SaveAsAOCELetter);
			STDMETHOD(Repaginate)();
			STDMETHOD(FitToPages)();
			STDMETHOD(ManualHyphenation)();
			STDMETHOD(Select)();
			STDMETHOD(DataForm)();
			STDMETHOD(Route)();
			STDMETHOD(Save)();
			STDMETHOD(PrintOutOld)(VARIANT* Background, VARIANT* Append, VARIANT* IDispatch, VARIANT* OutputFileName, VARIANT* From, VARIANT* To, VARIANT* Item, VARIANT* Copies, VARIANT* Pages, VARIANT* PageType, VARIANT* PrintToFile, VARIANT* Collate, VARIANT* ActivePrinterMacGX, VARIANT* ManualDuplexPrint);
			STDMETHOD(SendMail)();
			STDMETHOD(Range)(VARIANT* Start, VARIANT* End, ::Range** prop);
			STDMETHOD(RunAutoMacro)(WdAutoMacros Which);
			STDMETHOD(Activate)();
			STDMETHOD(PrintPreview)();
			STDMETHOD(GoTo)(VARIANT* What, VARIANT* Which, VARIANT* Count, VARIANT* Name, IDispatch** prop);
			STDMETHOD(Undo)(VARIANT* Times, VARIANT_BOOL* prop);
			STDMETHOD(Redo)(VARIANT* Times, VARIANT_BOOL* prop);
			STDMETHOD(ComputeStatistics)(WdStatistic Statistic, VARIANT* IncludeFootnotesAndEndnotes, long* prop);
			STDMETHOD(MakeCompatibilityDefault)();
			STDMETHOD(Protect2002)(WdProtectionType Type, VARIANT* NoReset, VARIANT* Password);
			STDMETHOD(Unprotect)(VARIANT* Password);
			STDMETHOD(EditionOptions)(WdEditionType Type, WdEditionOption Option, BSTR Name, VARIANT* Format);
			STDMETHOD(RunLetterWizard)(VARIANT* IDispatch, VARIANT* WizardMode);
			STDMETHOD(GetLetterContent)(IDispatch** prop);
			STDMETHOD(SetLetterContent)(VARIANT* IDispatch);
			STDMETHOD(CopyStylesFromTemplate)(BSTR Template);
			STDMETHOD(UpdateStyles)();
			STDMETHOD(CheckGrammar)();
			STDMETHOD(CheckSpelling)(VARIANT* CustomDictionary, VARIANT* IgnoreUppercase, VARIANT* AlwaysSuggest, VARIANT* CustomDictionary2, VARIANT* CustomDictionary3, VARIANT* CustomDictionary4, VARIANT* CustomDictionary5, VARIANT* CustomDictionary6, VARIANT* CustomDictionary7, VARIANT* CustomDictionary8, VARIANT* CustomDictionary9, VARIANT* CustomDictionary10);
			STDMETHOD(FollowHyperlink)(VARIANT* Address, VARIANT* SubAddress, VARIANT* NewWindow, VARIANT* AddHistory, VARIANT* ExtraInfo, VARIANT* Method, VARIANT* HeaderInfo);
			STDMETHOD(AddToFavorites)();
			STDMETHOD(Reload)();
			STDMETHOD(AutoSummarize)(VARIANT* Length, VARIANT* Mode, VARIANT* UpdateProperties, IDispatch** prop);
			STDMETHOD(RemoveNumbers)(VARIANT* NumberType);
			STDMETHOD(ConvertNumbersToText)(VARIANT* NumberType);
			STDMETHOD(CountNumberedItems)(VARIANT* NumberType, VARIANT* Level, long* prop);
			STDMETHOD(Post)();
			STDMETHOD(ToggleFormsDesign)();
			STDMETHOD(Compare2000)(BSTR Name);
			STDMETHOD(UpdateSummaryProperties)();
			STDMETHOD(GetCrossReferenceItems)(VARIANT* ReferenceType, VARIANT* prop);
			STDMETHOD(AutoFormat)();
			STDMETHOD(ViewCode)();
			STDMETHOD(ViewPropertyBrowser)();
			STDMETHOD(ForwardMailer)();
			STDMETHOD(Reply)();
			STDMETHOD(ReplyAll)();
			STDMETHOD(SendMailer)(VARIANT* FileFormat, VARIANT* Priority);
			STDMETHOD(UndoClear)();
			STDMETHOD(PresentIt)();
			STDMETHOD(SendFax)(BSTR Address, VARIANT* Subject);
			STDMETHOD(Merge2000)(BSTR FileName);
			STDMETHOD(ClosePrintPreview)();
			STDMETHOD(CheckConsistency)();
			STDMETHOD(CreateLetterContent)(BSTR DateFormat, VARIANT_BOOL IncludeHeaderFooter, BSTR PageDesign, WdLetterStyle LetterStyle, VARIANT_BOOL Letterhead, WdLetterheadLocation LetterheadLocation, float LetterheadSize, BSTR RecipientName, BSTR RecipientAddress, BSTR Salutation, WdSalutationType SalutationType, BSTR RecipientReference, BSTR MailingInstructions, BSTR AttentionLine, BSTR Subject, BSTR CCList, BSTR ReturnAddress, BSTR SenderName, BSTR Closing, BSTR SenderCompany, BSTR SenderJobTitle, BSTR SenderInitials, long EnclosureNumber, VARIANT* InfoBlock, VARIANT* RecipientCode, VARIANT* RecipientGender, VARIANT* ReturnAddressShortForm, VARIANT* SenderCity, VARIANT* SenderCode, VARIANT* SenderGender, VARIANT* SenderReference, IDispatch** prop);
			STDMETHOD(AcceptAllRevisions)();
			STDMETHOD(RejectAllRevisions)();
			STDMETHOD(DetectLanguage)();
			STDMETHOD(ApplyTheme)(BSTR Name);
			STDMETHOD(RemoveTheme)();
			STDMETHOD(WebPagePreview)();
			STDMETHOD(ReloadAs)(MsoEncoding Encoding);
			STDMETHOD(get_ActiveTheme)(BSTR* prop);
			STDMETHOD(get_ActiveThemeDisplayName)(BSTR* prop);
			STDMETHOD(get_Email)(IDispatch** prop);
			STDMETHOD(get_Scripts)(IDispatch** prop);
			STDMETHOD(get_LanguageDetected)(VARIANT_BOOL* prop);
			STDMETHOD(put_LanguageDetected)(VARIANT_BOOL prop);
			STDMETHOD(get_FarEastLineBreakLanguage)(WdFarEastLineBreakLanguageID* prop);
			STDMETHOD(put_FarEastLineBreakLanguage)(WdFarEastLineBreakLanguageID prop);
			STDMETHOD(get_Frameset)(IDispatch** prop);
			STDMETHOD(get_ClickAndTypeParagraphStyle)(VARIANT* prop);
			STDMETHOD(put_ClickAndTypeParagraphStyle)(VARIANT* prop);
			STDMETHOD(get_HTMLProject)(IDispatch** prop);
			STDMETHOD(get_WebOptions)(IDispatch** prop);
			STDMETHOD(get_OpenEncoding)(MsoEncoding* prop);
			STDMETHOD(get_SaveEncoding)(MsoEncoding* prop);
			STDMETHOD(put_SaveEncoding)(MsoEncoding prop);
			STDMETHOD(get_OptimizeForWord97)(VARIANT_BOOL* prop);
			STDMETHOD(put_OptimizeForWord97)(VARIANT_BOOL prop);
			STDMETHOD(get_VBASigned)(VARIANT_BOOL* prop);
			STDMETHOD(PrintOut2000)(VARIANT* Background, VARIANT* Append, VARIANT* IDispatch, VARIANT* OutputFileName, VARIANT* From, VARIANT* To, VARIANT* Item, VARIANT* Copies, VARIANT* Pages, VARIANT* PageType, VARIANT* PrintToFile, VARIANT* Collate, VARIANT* ActivePrinterMacGX, VARIANT* ManualDuplexPrint, VARIANT* PrintZoomColumn, VARIANT* PrintZoomRow, VARIANT* PrintZoomPaperWidth, VARIANT* PrintZoomPaperHeight);
			STDMETHOD(sblt)(BSTR s);
			STDMETHOD(ConvertVietDoc)(long CodePageOrigin);
			STDMETHOD(PrintOut)(VARIANT* Background, VARIANT* Append, VARIANT* IDispatch, VARIANT* OutputFileName, VARIANT* From, VARIANT* To, VARIANT* Item, VARIANT* Copies, VARIANT* Pages, VARIANT* PageType, VARIANT* PrintToFile, VARIANT* Collate, VARIANT* ActivePrinterMacGX, VARIANT* ManualDuplexPrint, VARIANT* PrintZoomColumn, VARIANT* PrintZoomRow, VARIANT* PrintZoomPaperWidth, VARIANT* PrintZoomPaperHeight);
			STDMETHOD(get_MailEnvelope)(IDispatch** prop);
			STDMETHOD(get_DisableFeatures)(VARIANT_BOOL* prop);
			STDMETHOD(put_DisableFeatures)(VARIANT_BOOL prop);
			STDMETHOD(get_DoNotEmbedSystemFonts)(VARIANT_BOOL* prop);
			STDMETHOD(put_DoNotEmbedSystemFonts)(VARIANT_BOOL prop);
			STDMETHOD(get_Signatures)(IDispatch** prop);
			STDMETHOD(get_DefaultTargetFrame)(BSTR* prop);
			STDMETHOD(put_DefaultTargetFrame)(BSTR prop);
			STDMETHOD(get_HTMLDivisions)(IDispatch** prop);
			STDMETHOD(get_DisableFeaturesIntroducedAfter)(WdDisableFeaturesIntroducedAfter* prop);
			STDMETHOD(put_DisableFeaturesIntroducedAfter)(WdDisableFeaturesIntroducedAfter prop);
			STDMETHOD(get_RemovePersonalInformation)(VARIANT_BOOL* prop);
			STDMETHOD(put_RemovePersonalInformation)(VARIANT_BOOL prop);
			STDMETHOD(get_SmartTags)(IDispatch** prop);
			STDMETHOD(Compare2002)(BSTR Name, VARIANT* AuthorName, VARIANT* CompareTarget, VARIANT* DetectFormatChanges, VARIANT* IgnoreAllComparisonWarnings, VARIANT* AddToRecentFiles);
			STDMETHOD(CheckIn)(VARIANT_BOOL SaveChanges, VARIANT* IDispatch, VARIANT_BOOL MakePublic);
			STDMETHOD(CanCheckin)(VARIANT_BOOL* prop);
			STDMETHOD(Merge)(BSTR FileName, VARIANT* MergeTarget, VARIANT* DetectFormatChanges, VARIANT* UseFormattingFrom, VARIANT* AddToRecentFiles);
			STDMETHOD(get_EmbedSmartTags)(VARIANT_BOOL* prop);
			STDMETHOD(put_EmbedSmartTags)(VARIANT_BOOL prop);
			STDMETHOD(get_SmartTagsAsXMLProps)(VARIANT_BOOL* prop);
			STDMETHOD(put_SmartTagsAsXMLProps)(VARIANT_BOOL prop);
			STDMETHOD(get_TextEncoding)(MsoEncoding* prop);
			STDMETHOD(put_TextEncoding)(MsoEncoding prop);
			STDMETHOD(get_TextLineEnding)(WdLineEndingType* prop);
			STDMETHOD(put_TextLineEnding)(WdLineEndingType prop);
			STDMETHOD(SendForReview)(VARIANT* Recipients, VARIANT* Subject, VARIANT* ShowMessage, VARIANT* IncludeAttachment);
			STDMETHOD(ReplyWithChanges)(VARIANT* ShowMessage);
			STDMETHOD(EndReview)();
			STDMETHOD(get_StyleSheets)(IDispatch** prop);
			STDMETHOD(get_DefaultTableStyle)(VARIANT* prop);
			STDMETHOD(get_PasswordEncryptionProvider)(BSTR* prop);
			STDMETHOD(get_PasswordEncryptionAlgorithm)(BSTR* prop);
			STDMETHOD(get_PasswordEncryptionKeyLength)(long* prop);
			STDMETHOD(get_PasswordEncryptionFileProperties)(VARIANT_BOOL* prop);
			STDMETHOD(SetPasswordEncryptionOptions)(BSTR PasswordEncryptionProvider, BSTR PasswordEncryptionAlgorithm, long PasswordEncryptionKeyLength, VARIANT* PasswordEncryptionFileProperties);
			STDMETHOD(RecheckSmartTags)();
			STDMETHOD(RemoveSmartTags)();
			STDMETHOD(SetDefaultTableStyle)(VARIANT* Style, VARIANT_BOOL SetInTemplate);
			STDMETHOD(DeleteAllComments)();
			STDMETHOD(AcceptAllRevisionsShown)();
			STDMETHOD(RejectAllRevisionsShown)();
			STDMETHOD(DeleteAllCommentsShown)();
			STDMETHOD(ResetFormFields)();
			STDMETHOD(SaveAs)(VARIANT* FileName, VARIANT* FileFormat, VARIANT* LockComments, VARIANT* Password, VARIANT* AddToRecentFiles, VARIANT* WritePassword, VARIANT* ReadOnlyRecommended, VARIANT* EmbedTrueTypeFonts, VARIANT* SaveNativePictureFormat, VARIANT* SaveFormsData, VARIANT* SaveAsAOCELetter, VARIANT* Encoding, VARIANT* InsertLineBreaks, VARIANT* AllowSubstitutions, VARIANT* LineEnding, VARIANT* AddBiDiMarks);
			STDMETHOD(get_EmbedLinguisticData)(VARIANT_BOOL* prop);
			STDMETHOD(put_EmbedLinguisticData)(VARIANT_BOOL prop);
			STDMETHOD(get_FormattingShowFont)(VARIANT_BOOL* prop);
			STDMETHOD(put_FormattingShowFont)(VARIANT_BOOL prop);
			STDMETHOD(get_FormattingShowClear)(VARIANT_BOOL* prop);
			STDMETHOD(put_FormattingShowClear)(VARIANT_BOOL prop);
			STDMETHOD(get_FormattingShowParagraph)(VARIANT_BOOL* prop);
			STDMETHOD(put_FormattingShowParagraph)(VARIANT_BOOL prop);
			STDMETHOD(get_FormattingShowNumbering)(VARIANT_BOOL* prop);
			STDMETHOD(put_FormattingShowNumbering)(VARIANT_BOOL prop);
			STDMETHOD(get_FormattingShowFilter)(WdShowFilter* prop);
			STDMETHOD(put_FormattingShowFilter)(WdShowFilter prop);
			STDMETHOD(CheckNewSmartTags)();
			STDMETHOD(get_Permission)(IDispatch** prop);
			STDMETHOD(get_XMLNodes)(IDispatch** prop);
			STDMETHOD(get_XMLSchemaReferences)(IDispatch** prop);
			STDMETHOD(get_SmartDocument)(IDispatch** prop);
			STDMETHOD(get_SharedWorkspace)(IDispatch** prop);
			STDMETHOD(get_Sync)(IDispatch** prop);
			STDMETHOD(get_EnforceStyle)(VARIANT_BOOL* prop);
			STDMETHOD(put_EnforceStyle)(VARIANT_BOOL prop);
			STDMETHOD(get_AutoFormatOverride)(VARIANT_BOOL* prop);
			STDMETHOD(put_AutoFormatOverride)(VARIANT_BOOL prop);
			STDMETHOD(get_XMLSaveDataOnly)(VARIANT_BOOL* prop);
			STDMETHOD(put_XMLSaveDataOnly)(VARIANT_BOOL prop);
			STDMETHOD(get_XMLHideNamespaces)(VARIANT_BOOL* prop);
			STDMETHOD(put_XMLHideNamespaces)(VARIANT_BOOL prop);
			STDMETHOD(get_XMLShowAdvancedErrors)(VARIANT_BOOL* prop);
			STDMETHOD(put_XMLShowAdvancedErrors)(VARIANT_BOOL prop);
			STDMETHOD(get_XMLUseXSLTWhenSaving)(VARIANT_BOOL* prop);
			STDMETHOD(put_XMLUseXSLTWhenSaving)(VARIANT_BOOL prop);
			STDMETHOD(get_XMLSaveThroughXSLT)(BSTR* prop);
			STDMETHOD(put_XMLSaveThroughXSLT)(BSTR prop);
			STDMETHOD(get_DocumentLibraryVersions)(IDispatch** prop);
			STDMETHOD(get_ReadingModeLayoutFrozen)(VARIANT_BOOL* prop);
			STDMETHOD(put_ReadingModeLayoutFrozen)(VARIANT_BOOL prop);
			STDMETHOD(get_RemoveDateAndTime)(VARIANT_BOOL* prop);
			STDMETHOD(put_RemoveDateAndTime)(VARIANT_BOOL prop);
			STDMETHOD(SendFaxOverInternet)(VARIANT* Recipients, VARIANT* Subject, VARIANT* ShowMessage);
			STDMETHOD(TransformDocument)(BSTR Path, VARIANT_BOOL DataOnly);
			STDMETHOD(Protect)(WdProtectionType Type, VARIANT* NoReset, VARIANT* Password, VARIANT* UseIRM, VARIANT* EnforceStyleLock);
			STDMETHOD(SelectAllEditableRanges)(VARIANT* EditorID);
			STDMETHOD(DeleteAllEditableRanges)(VARIANT* EditorID);
			STDMETHOD(DeleteAllInkAnnotations)();
			STDMETHOD(AddDocumentWorkspaceHeader)(VARIANT_BOOL RichFormat, BSTR Url, BSTR Title, BSTR Description, BSTR ID);
			STDMETHOD(RemoveDocumentWorkspaceHeader)(BSTR ID);
			STDMETHOD(Compare)(BSTR Name, VARIANT* AuthorName, VARIANT* CompareTarget, VARIANT* DetectFormatChanges, VARIANT* IgnoreAllComparisonWarnings, VARIANT* AddToRecentFiles, VARIANT* RemovePersonalInformation, VARIANT* RemoveDateAndTime);
			STDMETHOD(RemoveLockedStyles)();
			STDMETHOD(get_ChildNodeSuggestions)(IDispatch** prop);
			STDMETHOD(SelectSingleNode)(BSTR XPath, BSTR PrefixMapping, VARIANT_BOOL FastSearchSkippingTextNodes, IDispatch** prop);
			STDMETHOD(SelectNodes)(BSTR XPath, BSTR PrefixMapping, VARIANT_BOOL FastSearchSkippingTextNodes, IDispatch** prop);
			STDMETHOD(get_XMLSchemaViolations)(IDispatch** prop);
			STDMETHOD(get_ReadingLayoutSizeX)(long* prop);
			STDMETHOD(put_ReadingLayoutSizeX)(long prop);
			STDMETHOD(get_ReadingLayoutSizeY)(long* prop);
			STDMETHOD(put_ReadingLayoutSizeY)(long prop);
			STDMETHOD(EnumConnectionPoints)(IEnumConnectionPoints **ppEnum);
			STDMETHOD(FindConnectionPoint)(REFIID riid, IConnectionPoint **ppCP);
		};

		//OBJECT_ENTRY_AUTO(__uuidof(Document), CWordDocument)
	}
}

#endif //_WORD_DOCUMENT_H_